@@ -20,11 +20,11 @@ Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA */
 
 #include "blargg_source.h"
 
-long const spectrum_clock = 3546900;
-long const cpc_clock      = 2000000;
+static long const spectrum_clock = 3546900;
+static long const cpc_clock      = 2000000;
 
-unsigned const ram_start = 0x4000;
-int const osc_count = Ay_Apu::osc_count + 1;
+static unsigned const ram_start = 0x4000;
+static int const osc_count = Ay_Apu::osc_count + 1;
 
 using std::min;
 using std::max;
@@ -33,12 +33,12 @@ Ay_Emu::Ay_Emu()
 {
 	beeper_output = 0;
 	set_type( gme_ay_type );
-	
+
 	static const char* const names [osc_count] = {
 		"Wave 1", "Wave 2", "Wave 3", "Beeper"
 	};
 	set_voice_names( names );
-	
+
 	static int const types [osc_count] = {
 		wave_type | 0, wave_type | 1, wave_type | 2, mixed_type | 0
 	};
@@ -66,18 +66,18 @@ static blargg_err_t parse_header( byte const* in, long size, Ay_Emu::file_t* out
 	typedef Ay_Emu::header_t header_t;
 	out->header = (header_t const*) in;
 	out->end    = in + size;
-	
+
 	if ( size < Ay_Emu::header_size )
 		return gme_wrong_file_type;
-	
+
 	header_t const& h = *(header_t const*) in;
 	if ( memcmp( h.tag, "ZXAYEMUL", 8 ) )
 		return gme_wrong_file_type;
-	
+
 	out->tracks = get_data( *out, h.track_info, (h.max_track + 1) * 4 );
 	if ( !out->tracks )
 		return "Missing track data";
-	
+
 	return 0;
 }
 
@@ -87,7 +87,7 @@ static void copy_ay_fields( Ay_Emu::file_t const& file, track_info_t* out, int t
 	byte const* track_info = get_data( file, file.tracks + track * 4 + 2, 6 );
 	if ( track_info )
 		out->length = get_be16( track_info + 4 ) * (1000L / 50); // frames to msec
-	
+
 	Gme_File::copy_field_( out->author,  (char const*) get_data( file, file.header->author, 1 ) );
 	Gme_File::copy_field_( out->comment, (char const*) get_data( file, file.header->comment, 1 ) );
 }
@@ -101,16 +101,16 @@ blargg_err_t Ay_Emu::track_info_( track_info_t* out, int track ) const
 struct Ay_File : Gme_Info_
 {
 	Ay_Emu::file_t file;
-	
+
 	Ay_File() { set_type( gme_ay_type ); }
-	
+
 	blargg_err_t load_mem_( byte const* begin, long size )
 	{
 		RETURN_ERR( parse_header( begin, size, &file ) );
 		set_track_count( file.header->max_track + 1 );
 		return 0;
 	}
-	
+
 	blargg_err_t track_info_( track_info_t* out, int track ) const
 	{
 		copy_ay_fields( file, out, track );
@@ -128,20 +128,20 @@ extern gme_type_t const gme_ay_type = &gme_ay_type_;
 
 blargg_err_t Ay_Emu::load_mem_( byte const* in, long size )
 {
-	assert( offsetof (header_t,track_info [2]) == header_size );
-	
+	blaarg_static_assert( offsetof (header_t,track_info [2]) == header_size, "AY Header layout incorrect!" );
+
 	RETURN_ERR( parse_header( in, size, &file ) );
 	set_track_count( file.header->max_track + 1 );
-	
+
 	if ( file.header->vers > 2 )
 		set_warning( "Unknown file version" );
-	
+
 	set_voice_count( osc_count );
 	apu.volume( gain() );
-	
+
 	return setup_buffer( spectrum_clock );
 }
-	
+
 void Ay_Emu::update_eq( blip_eq_t const& eq )
 {
 	apu.treble_eq( eq );
@@ -165,23 +165,23 @@ void Ay_Emu::set_tempo_( double t )
 blargg_err_t Ay_Emu::start_track_( int track )
 {
 	RETURN_ERR( Classic_Emu::start_track_( track ) );
-	
+
 	memset( mem.ram + 0x0000, 0xC9, 0x100 ); // fill RST vectors with RET
 	memset( mem.ram + 0x0100, 0xFF, 0x4000 - 0x100 );
 	memset( mem.ram + ram_start, 0x00, sizeof mem.ram - ram_start );
 	memset( mem.padding1, 0xFF, sizeof mem.padding1 );
 	memset( mem.ram + 0x10000, 0xFF, sizeof mem.ram - 0x10000 );
-	
+
 	// locate data blocks
 	byte const* const data = get_data( file, file.tracks + track * 4 + 2, 14 );
 	if ( !data ) return "File data missing";
-	
+
 	byte const* const more_data = get_data( file, data + 10, 6 );
 	if ( !more_data ) return "File data missing";
-	
+
 	byte const* blocks = get_data( file, data + 12, 8 );
 	if ( !blocks ) return "File data missing";
-	
+
 	// initial addresses
 	cpu::reset( mem.ram );
 	r.sp = get_be16( more_data );
@@ -189,14 +189,14 @@ blargg_err_t Ay_Emu::start_track_( int track )
 	r.b.flags = r.b.c = r.b.e = r.b.l = data [9];
 	r.alt.w = r.w;
 	r.ix = r.iy = r.w.hl;
-	
+
 	unsigned addr = get_be16( blocks );
 	if ( !addr ) return "File data missing";
-	
+
 	unsigned init = get_be16( more_data + 2 );
 	if ( !init )
 		init = addr;
-	
+
 	// copy blocks into memory
 	do
 	{
@@ -218,7 +218,7 @@ blargg_err_t Ay_Emu::start_track_( int track )
 		if ( addr < ram_start && addr >= 0x400 ) // several tracks use low data
 			debug_printf( "Block addr in ROM\n" );
 		memcpy( mem.ram + addr, in, len );
-		
+
 		if ( file.end - blocks < 8 )
 		{
 			set_warning( "Missing file data" );
@@ -226,7 +226,7 @@ blargg_err_t Ay_Emu::start_track_( int track )
 		}
 	}
 	while ( (addr = get_be16( blocks )) != 0 );
-	
+
 	// copy and configure driver
 	static byte const passive [] = {
 		0xF3,       // DI
@@ -256,24 +256,24 @@ blargg_err_t Ay_Emu::start_track_( int track )
 	}
 	mem.ram [2] = init;
 	mem.ram [3] = init >> 8;
-	
+
 	mem.ram [0x38] = 0xFB; // Put EI at interrupt vector (followed by RET)
-	
+
 	memcpy( mem.ram + 0x10000, mem.ram, 0x80 ); // some code wraps around (ugh)
-	
+
 	beeper_delta = int (apu.amp_range * 0.65);
 	last_beeper = 0;
 	apu.reset();
 	next_play = play_period;
-	
+
 	// start at spectrum speed
 	change_clock_rate( spectrum_clock );
 	set_tempo( tempo() );
-	
+
 	spectrum_mode = false;
 	cpc_mode      = false;
 	cpc_latch     = 0;
-	
+
 	return 0;
 }
 
@@ -289,14 +289,14 @@ void Ay_Emu::cpu_out_misc( cpu_time_t time, unsigned addr, int data )
 			spectrum_mode = true;
 			apu_addr = data & 0x0F;
 			return;
-		
+
 		case 0xBEFD:
 			spectrum_mode = true;
 			apu.write( time, apu_addr, data );
 			return;
 		}
 	}
-	
+
 	if ( !spectrum_mode )
 	{
 		switch ( addr >> 8 )
@@ -307,22 +307,22 @@ void Ay_Emu::cpu_out_misc( cpu_time_t time, unsigned addr, int data )
 			case 0xC0:
 				apu_addr = cpc_latch & 0x0F;
 				goto enable_cpc;
-			
+
 			case 0x80:
 				apu.write( time, apu_addr, cpc_latch );
 				goto enable_cpc;
 			}
 			break;
-		
+
 		case 0xF4:
 			cpc_latch = data;
 			goto enable_cpc;
 		}
 	}
-	
+
 	debug_printf( "Unmapped OUT: $%04X <- $%02X\n", addr, data );
 	return;
-	
+
 enable_cpc:
 	if ( !cpc_mode )
 	{
@@ -335,7 +335,7 @@ enable_cpc:
 void ay_cpu_out( Ay_Cpu* cpu, cpu_time_t time, unsigned addr, int data )
 {
 	Ay_Emu& emu = STATIC_CAST(Ay_Emu&,*cpu);
-	
+
 	if ( (addr & 0xFF) == 0xFE && !emu.cpc_mode )
 	{
 		int delta = emu.beeper_delta;
@@ -360,7 +360,7 @@ int ay_cpu_in( Ay_Cpu*, unsigned addr )
 	// keyboard read and other things
 	if ( (addr & 0xFF) == 0xFE )
 		return 0xFF; // other values break some beeper tunes
-	
+
 	debug_printf( "Unmapped IN : $%04X\n", addr );
 	return 0xFF;
 }
@@ -370,22 +370,22 @@ blargg_err_t Ay_Emu::run_clocks( blip_time_t& duration, int )
 	set_time( 0 );
 	if ( !(spectrum_mode | cpc_mode) )
 		duration /= 2; // until mode is set, leave room for halved clock rate
-	
+
 	while ( time() < duration )
 	{
 		cpu::run( min( duration, (blip_time_t) next_play ) );
-		
+
 		if ( time() >= next_play )
 		{
 			next_play += play_period;
-			
+
 			if ( r.iff1 )
 			{
 				if ( mem.ram [r.pc] == 0x76 )
 					r.pc++;
-				
+
 				r.iff1 = r.iff2 = 0;
-				
+
 				mem.ram [--r.sp] = uint8_t (r.pc >> 8);
 				mem.ram [--r.sp] = uint8_t (r.pc);
 				r.pc = 0x38;
@@ -403,8 +403,8 @@ blargg_err_t Ay_Emu::run_clocks( blip_time_t& duration, int )
 	next_play -= duration;
 	check( next_play >= 0 );
 	adjust_time( -duration );
-	
+
 	apu.end_frame( duration );
-	
+
 	return 0;
 }
