@@ -1,6 +1,6 @@
 // Sinclair Spectrum AY music file emulator
 
-// Game_Music_Emu 0.5.5
+// Game_Music_Emu https://bitbucket.org/mpyne/game-music-emu/
 #ifndef AY_EMU_H
 #define AY_EMU_H
 
@@ -25,7 +25,7 @@ public:
 		byte first_track;
 		byte track_info [2];
 	};
-	
+
 	static gme_type_t static_type() { return gme_ay_type; }
 public:
 	Ay_Emu();
@@ -45,8 +45,7 @@ protected:
 	void update_eq( blip_eq_t const& );
 private:
 	file_t file;
-	
-	unsigned play_addr;
+
 	cpu_time_t play_period;
 	cpu_time_t next_play;
 	Blip_Buffer* beeper_output;
@@ -56,7 +55,7 @@ private:
 	int cpc_latch;
 	bool spectrum_mode;
 	bool cpc_mode;
-	
+
 	// large items
 	struct {
 		byte padding1 [0x100];
