@@ -26,21 +26,21 @@ Gbs_Emu::equalizer_t const Gbs_Emu::headphones_eq =
 Gbs_Emu::Gbs_Emu()
 {
 	set_type( gme_gbs_type );
-	
+
 	static const char* const names [Gb_Apu::osc_count] = {
 		"Square 1", "Square 2", "Wave", "Noise"
 	};
 	set_voice_names( names );
-	
+
 	static int const types [Gb_Apu::osc_count] = {
 		wave_type | 1, wave_type | 2, wave_type | 0, mixed_type | 0
 	};
 	set_voice_types( types );
-	
+
 	set_silence_lookahead( 6 );
 	set_max_initial_silence( 21 );
 	set_gain( 1.2 );
-	
+
 	set_equalizer( make_equalizer( -1.0, 120 ) );
 }
 
@@ -77,19 +77,19 @@ static blargg_err_t check_gbs_header( void const* header )
 struct Gbs_File : Gme_Info_
 {
 	Gbs_Emu::header_t h;
-	
+
 	Gbs_File() { set_type( gme_gbs_type ); }
-	
+
 	blargg_err_t load_( Data_Reader& in )
 	{
 		blargg_err_t err = in.read( &h, Gbs_Emu::header_size );
 		if ( err )
 			return (err == in.eof_error ? gme_wrong_file_type : err);
-		
+
 		set_track_count( h.track_count );
 		return check_gbs_header( &h );
 	}
-	
+
 	blargg_err_t track_info_( track_info_t* out, int ) const
 	{
 		copy_gbs_fields( h, out );
@@ -107,27 +107,27 @@ extern gme_type_t const gme_gbs_type = &gme_gbs_type_;
 
 blargg_err_t Gbs_Emu::load_( Data_Reader& in )
 {
-	assert( offsetof (header_t,copyright [32]) == header_size );
+	blaarg_static_assert( offsetof (header_t,copyright [32]) == header_size, "GBS Header layout incorrect!" );
 	RETURN_ERR( rom.load( in, header_size, &header_, 0 ) );
-	
+
 	set_track_count( header_.track_count );
 	RETURN_ERR( check_gbs_header( &header_ ) );
-	
+
 	if ( header_.vers != 1 )
 		set_warning( "Unknown file version" );
-	
+
 	if ( header_.timer_mode & 0x78 )
 		set_warning( "Invalid timer mode" );
-	
+
 	unsigned load_addr = get_le16( header_.load_addr );
 	if ( (header_.load_addr [1] | header_.init_addr [1] | header_.play_addr [1]) > 0x7F ||
 			load_addr < 0x400 )
 		set_warning( "Invalid load/init/play address" );
-	
+
 	set_voice_count( Gb_Apu::osc_count );
-	
+
 	apu.volume( gain() );
-	
+
 	return setup_buffer( 4194304 );
 }
 
@@ -147,16 +147,19 @@ void Gbs_Emu::set_voice( int i, Blip_Buffer* c, Blip_Buffer* l, Blip_Buffer* r )
 
 void Gbs_Emu::set_bank( int n )
 {
-	blargg_long addr = rom.mask_addr( n * (blargg_long) bank_size );
-	if ( addr == 0 && rom.size() > bank_size )
+	// Only valid for MBC1 cartridges, but hopefully shouldn't hurt
+	n &= 0x1f;
+	if (n == 0)
 	{
-		// TODO: what is the correct behavior? Current Game & Watch Gallery
-		// rip requires that this have no effect or set to bank 1.
-		//debug_printf( "Selected ROM bank 0\n" );
-		return;
-		//n = 1;
+		n = 1;
 	}
-	cpu::map_code( bank_size, bank_size, rom.at_addr( addr ) );
+
+	blargg_long addr = n * (blargg_long) bank_size;
+	if (addr > rom.size())
+	{
+		return;
+	}
+	cpu::map_code( bank_size, bank_size, rom.at_addr( rom.mask_addr( addr ) ) );
 }
 
 void Gbs_Emu::update_timer()
@@ -203,37 +206,37 @@ void Gbs_Emu::set_tempo_( double t )
 blargg_err_t Gbs_Emu::start_track_( int track )
 {
 	RETURN_ERR( Classic_Emu::start_track_( track ) );
-	
+
 	memset( ram, 0, 0x4000 );
-	memset( ram + 0x4000, 0xFF, 0x1F80 );
-	memset( ram + 0x5F80, 0, sizeof ram - 0x5F80 );
+	memset( ram + 0x4000, 0xFF, 0x1F00 );
+	memset( ram + 0x5F00, 0, sizeof ram - 0x5F00 );
 	ram [hi_page] = 0; // joypad reads back as 0
-	
+
 	apu.reset();
 	for ( int i = 0; i < (int) sizeof sound_data; i++ )
 		apu.write_register( 0, i + apu.start_addr, sound_data [i] );
-	
+
 	unsigned load_addr = get_le16( header_.load_addr );
 	rom.set_addr( load_addr );
 	cpu::rst_base = load_addr;
-	
+
 	cpu::reset( rom.unmapped() );
-	
+
 	cpu::map_code( ram_addr, 0x10000 - ram_addr, ram );
 	cpu::map_code( 0, bank_size, rom.at_addr( 0 ) );
 	set_bank( rom.size() > bank_size );
-	
+
 	ram [hi_page + 6] = header_.timer_modulo;
 	ram [hi_page + 7] = header_.timer_mode;
 	update_timer();
 	next_play = play_period;
-	
+
 	cpu::r.a  = track;
 	cpu::r.pc = idle_addr;
 	cpu::r.sp = get_le16( header_.stack_ptr );
 	cpu_time  = 0;
 	cpu_jsr( get_le16( header_.init_addr ) );
-	
+
 	return 0;
 }
 
@@ -246,7 +249,7 @@ blargg_err_t Gbs_Emu::run_clocks( blip_time_t& duration, int )
 		cpu_time = duration;
 		bool result = cpu::run( count );
 		cpu_time -= cpu::remain();
-		
+
 		if ( result )
 		{
 			if ( cpu::r.pc == idle_addr )
@@ -256,7 +259,7 @@ blargg_err_t Gbs_Emu::run_clocks( blip_time_t& duration, int )
 					cpu_time = duration;
 					break;
 				}
-				
+
 				if ( cpu_time < next_play )
 					cpu_time = next_play;
 				next_play += play_period;
@@ -279,12 +282,12 @@ blargg_err_t Gbs_Emu::run_clocks( blip_time_t& duration, int )
 			}
 		}
 	}
-	
+
 	duration = cpu_time;
 	next_play -= cpu_time;
 	if ( next_play < 0 ) // could go negative if routine is taking too long to return
 		next_play = 0;
 	apu.end_frame( cpu_time );
-	
+
 	return 0;
 }
