@@ -1,6 +1,6 @@
 // Turbo Grafx 16 (PC Engine) PSG sound chip emulator
 
-// Game_Music_Emu 0.5.5
+// Game_Music_Emu https://bitbucket.org/mpyne/game-music-emu/
 #ifndef HES_APU_H
 #define HES_APU_H
 
@@ -19,15 +19,15 @@ struct Hes_Osc
 	unsigned char balance;
 	unsigned char dac;
 	blip_time_t last_time;
-	
+
 	Blip_Buffer* outputs [2];
 	Blip_Buffer* chans [3];
 	unsigned noise_lfsr;
 	unsigned char control;
-	
-	enum { amp_range = 0x8000 };
+
+	static const unsigned int amp_range = 0x8000;
 	typedef Blip_Synth<blip_med_quality,1> synth_t;
-	
+
 	void run_until( synth_t& synth, blip_time_t );
 };
 
@@ -35,18 +35,18 @@ class Hes_Apu {
 public:
 	void treble_eq( blip_eq_t const& );
 	void volume( double );
-	
-	enum { osc_count = 6 };
+
+	static const int osc_count = 6;
 	void osc_output( int index, Blip_Buffer* center, Blip_Buffer* left, Blip_Buffer* right );
-	
+
 	void reset();
-	
-	enum { start_addr = 0x0800 };
-	enum { end_addr   = 0x0809 };
+
+	static const unsigned int start_addr = 0x0800;
+	static const unsigned int end_addr   = 0x0809;
 	void write_data( blip_time_t, int addr, int data );
-	
+
 	void end_frame( blip_time_t );
-	
+
 public:
 	Hes_Apu();
 private:
@@ -54,7 +54,7 @@ private:
 	int latch;
 	int balance;
 	Hes_Osc::synth_t synth;
-	
+
 	void balance_changed( Hes_Osc& );
 	void recalc_chans();
 };
